@@ -1,11 +1,10 @@
 use std::{
 	borrow::Cow,
 	collections::HashMap,
-	env::current_dir,
+	env::var,
 	ffi::{OsStr, OsString},
 	fs::File,
 	io::{IsTerminal, Write},
-	path::Path,
 	process::Stdio,
 	sync::{
 		atomic::{AtomicBool, AtomicU8, Ordering},
@@ -32,7 +31,7 @@ use watchexec_events::{Event, Keyboard, ProcessEnd, Tag};
 use watchexec_signals::Signal;
 
 use crate::{
-	args::{Args, ClearMode, ColourMode, EmitEvents, OnBusyUpdate, SignalMapping},
+	args::{Args, ClearMode, ColourMode, EmitEvents, OnBusyUpdate, SignalMapping, WrapMode},
 	state::RotatingTempFile,
 };
 use crate::{emits::events_to_simple_format, state::State};
@@ -68,19 +67,7 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 		eprintln!("[[Error (not fatal)]]\n{}", Report::new(err.error));
 	});
 
-	config.pathset(if args.paths.is_empty() {
-		vec![current_dir().into_diagnostic()?]
-	} else if args.paths.len() == 1
-		&& args
-			.paths
-			.first()
-			.map_or(false, |p| p == Path::new("/dev/null"))
-	{
-		// special case: /dev/null means "don't start the fs event source"
-		Vec::new()
-	} else {
-		args.paths.clone()
-	});
+	config.pathset(args.paths.clone());
 
 	config.throttle(args.debounce.0);
 	config.keyboard_events(args.stdin_quit);
@@ -114,15 +101,7 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 						clearscreen::clear().ok();
 					}
 					ClearMode::Reset => {
-						for cs in [
-							ClearScreen::WindowsCooked,
-							ClearScreen::WindowsVt,
-							ClearScreen::VtLeaveAlt,
-							ClearScreen::VtWellDone,
-							ClearScreen::default(),
-						] {
-							cs.clear().ok();
-						}
+						reset_screen();
 					}
 				}
 			}
@@ -232,17 +211,52 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 
 					if let Some(ref workdir) = workdir.as_ref() {
 						debug!(?workdir, "set command workdir");
-						command.current_dir(workdir);
+						command.command_mut().current_dir(workdir);
 					}
 
-					emit_events_to_command(command, events, emit_file, emit_events_to, add_envs);
+					emit_events_to_command(
+						command.command_mut(),
+						events,
+						emit_file,
+						emit_events_to,
+						add_envs,
+					);
 				});
 
-				let show_events = || {
-					if print_events {
-						trace!("print events to stderr");
-						for (n, event) in action.events.iter().enumerate() {
-							eprintln!("[EVENT {n}] {event}");
+				let show_events = {
+					let events = action.events.clone();
+					move || {
+						if print_events {
+							trace!("print events to stderr");
+							for (n, event) in events.iter().enumerate() {
+								eprintln!("[EVENT {n}] {event}");
+							}
+						}
+					}
+				};
+
+				let clear_screen = {
+					let events = action.events.clone();
+					move || {
+						if let Some(mode) = clear {
+							match mode {
+								ClearMode::Clear => {
+									clearscreen::clear().ok();
+									debug!("cleared screen");
+								}
+								ClearMode::Reset => {
+									reset_screen();
+									debug!("hard-reset screen");
+								}
+							}
+						}
+
+						// re-show events after clearing
+						if print_events {
+							trace!("print events to stderr");
+							for (n, event) in events.iter().enumerate() {
+								eprintln!("[EVENT {n}] {event}");
+							}
 						}
 					}
 				};
@@ -335,28 +349,6 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 					return action;
 				}
 
-				// clear the screen before printing events
-				if let Some(mode) = clear {
-					match mode {
-						ClearMode::Clear => {
-							clearscreen::clear().ok();
-							debug!("cleared screen");
-						}
-						ClearMode::Reset => {
-							for cs in [
-								ClearScreen::WindowsCooked,
-								ClearScreen::WindowsVt,
-								ClearScreen::VtLeaveAlt,
-								ClearScreen::VtWellDone,
-								ClearScreen::default(),
-							] {
-								cs.clear().ok();
-							}
-							debug!("hard-reset screen");
-						}
-					}
-				}
-
 				show_events();
 
 				if let Some(delay) = delay_run {
@@ -390,6 +382,7 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 									OnBusyUpdate::Restart if cfg!(windows) => {
 										job.restart();
 										job.run(move |context| {
+											clear_screen();
 											setup_process(
 												innerjob.clone(),
 												context.command.clone(),
@@ -403,6 +396,7 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 											stop_timeout,
 										);
 										job.run(move |context| {
+											clear_screen();
 											setup_process(
 												innerjob.clone(),
 												context.command.clone(),
@@ -426,6 +420,7 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 													trace!("job finished, starting queued");
 													job.start();
 													job.run(move |context| {
+														clear_screen();
 														setup_process(
 															innerjob.clone(),
 															context.command.clone(),
@@ -444,6 +439,7 @@ pub fn make_config(args: &Args, state: &State) -> Result<Config> {
 								trace!("job is not running, start it");
 								job.start();
 								job.run(move |context| {
+									clear_screen();
 									setup_process(
 										innerjob.clone(),
 										context.command.clone(),
@@ -471,35 +467,52 @@ fn interpret_command_args(args: &Args) -> Result<Arc<Command>> {
 		panic!("(clap) Bug: command is not present");
 	}
 
-	let shell = match if args.no_shell || args.no_shell_long {
+	let shell = if args.no_shell {
 		None
 	} else {
-		args.shell.as_deref().or(Some("default"))
-	} {
-		Some("") => return Err(RuntimeError::CommandShellEmptyShell).into_diagnostic(),
-
-		Some("none") | None => None,
-
-		#[cfg(windows)]
-		Some("default") | Some("cmd") | Some("cmd.exe") | Some("CMD") | Some("CMD.EXE") => {
-			Some(Shell::cmd())
-		}
-
-		#[cfg(not(windows))]
-		Some("default") => Some(Shell::new("sh")),
-
-		Some(other) => {
-			let sh = other.split_ascii_whitespace().collect::<Vec<_>>();
-
-			// UNWRAP: checked by Some("")
-			#[allow(clippy::unwrap_used)]
-			let (shprog, shopts) = sh.split_first().unwrap();
-
-			Some(Shell {
-				prog: shprog.into(),
-				options: shopts.iter().map(|s| (*s).to_string()).collect(),
-				program_option: Some(Cow::Borrowed(OsStr::new("-c"))),
+		let shell = args.shell.clone().or_else(|| var("SHELL").ok());
+		match shell
+			.as_deref()
+			.or_else(|| {
+				if cfg!(not(windows)) {
+					Some("sh")
+				} else if var("POWERSHELL_DISTRIBUTION_CHANNEL").is_ok()
+					&& (which::which("pwsh").is_ok() || which::which("pwsh.exe").is_ok())
+				{
+					trace!("detected pwsh");
+					Some("pwsh")
+				} else if var("PSModulePath").is_ok()
+					&& (which::which("powershell").is_ok()
+						|| which::which("powershell.exe").is_ok())
+				{
+					trace!("detected powershell");
+					Some("powershell")
+				} else {
+					Some("cmd")
+				}
 			})
+			.or(Some("default"))
+		{
+			Some("") => return Err(RuntimeError::CommandShellEmptyShell).into_diagnostic(),
+
+			Some("none") | None => None,
+
+			#[cfg(windows)]
+			Some("cmd") | Some("cmd.exe") | Some("CMD") | Some("CMD.EXE") => Some(Shell::cmd()),
+
+			Some(other) => {
+				let sh = other.split_ascii_whitespace().collect::<Vec<_>>();
+
+				// UNWRAP: checked by Some("")
+				#[allow(clippy::unwrap_used)]
+				let (shprog, shopts) = sh.split_first().unwrap();
+
+				Some(Shell {
+					prog: shprog.into(),
+					options: shopts.iter().map(|s| (*s).to_string()).collect(),
+					program_option: Some(Cow::Borrowed(OsStr::new("-c"))),
+				})
+			}
 		}
 	};
 
@@ -519,7 +532,8 @@ fn interpret_command_args(args: &Args) -> Result<Arc<Command>> {
 	Ok(Arc::new(Command {
 		program,
 		options: SpawnOptions {
-			grouped: !args.no_process_group,
+			grouped: matches!(args.wrap_process, WrapMode::Group),
+			session: matches!(args.wrap_process, WrapMode::Session),
 			..Default::default()
 		},
 	}))
@@ -680,3 +694,15 @@ fn emit_events_to_command(
 		command.stdin(stdin);
 	}
 }
+
+pub(crate) fn reset_screen() {
+	for cs in [
+		ClearScreen::WindowsCooked,
+		ClearScreen::WindowsVt,
+		ClearScreen::VtLeaveAlt,
+		ClearScreen::VtWellDone,
+		ClearScreen::default(),
+	] {
+		cs.clear().ok();
+	}
+}
